"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.EncodeCtx = void 0;
const interface_1 = require("../interface");
const utils_1 = require("../utils");
const data_1 = require("../data");
const utils_2 = require("../encodings/utils");
const fixed_1 = require("../encodings/fixed");
const unrec_symbol_1 = require("../encodings/unrec-symbol");
class EncodeCtx {
    constructor(_store) {
        this._store = _store;
        this._encodingKeys = new Map();
        this._objectToRef = new Map();
        this._encodingSpec = Object.create(null);
        this._metadata = Object.create(null);
        this._workingMessage = [0];
        this._isImplicit = false;
        this.metadata = undefined;
    }
    _getEncodingIndexAndRegister(encoding) {
        if (encoding[interface_1.fixedIndexProp] != null) {
            return encoding[interface_1.fixedIndexProp];
        }
        const encodingKeys = this._encodingKeys;
        let encodingIndex = encodingKeys.get(encoding);
        if (encodingIndex == null) {
            encodingIndex = encodingKeys.size + fixed_1.Fixed.End;
            encodingKeys.set(encoding, encodingIndex);
        }
        return encodingIndex;
    }
    _mustGetBySymbol(s) {
        const inner = this._store.mayGetBySymbol(s);
        if (!inner) {
            return (0, unrec_symbol_1.getUnrecSymbolEncoding)(s);
        }
        return inner;
    }
    _createNewRef(value) {
        const { _workingMessage: msg, _objectToRef, _encodingSpec, _metadata, _store } = this;
        const index = msg.length;
        const ref = `${index}`;
        _objectToRef.set(value, ref);
        if (typeof value === "string") {
            msg.push(value);
            return ref;
        }
        msg.push(0);
        if (typeof value === "symbol") {
            const encoding = this._mustGetBySymbol(value);
            this._encodingSpec[index] =
                this._getEncodingIndexAndRegister(encoding);
            if ("metadata" in encoding && encoding.metadata) {
                this._metadata[index] = encoding.metadata;
            }
            return ref;
        }
        const encoding = _store.mustGetByProto(value);
        const oldMetadata = this.metadata;
        const preszed = encoding.encode(value, this);
        // _isImplicit will be set only on specific library encodings.
        // For example `object`. This is to make sure regular objects don't get the extra
        // characters needed to mark their encodings.
        if (!this._isImplicit) {
            _encodingSpec[index] = this._getEncodingIndexAndRegister(encoding);
        }
        this._isImplicit = false;
        if (this.metadata !== undefined) {
            _metadata[index] = this.metadata;
        }
        this.metadata = oldMetadata;
        msg[index] = preszed;
        return ref;
    }
    _makeEncodingKeys() {
        const encodingKeys = [];
        // We're just using the fact Maps are sorted by insertion order
        for (const encoding of this._encodingKeys.keys()) {
            encodingKeys.push((0, utils_2.getEncodingKey)(encoding));
        }
        return encodingKeys;
    }
    encode(value) {
        const tryScalar = (0, data_1.tryEncodeScalar)(value);
        if (tryScalar !== data_1.noResultPlaceholder) {
            return tryScalar;
        }
        let foundRef = this._objectToRef.get(value);
        if (!foundRef) {
            foundRef = this._createNewRef(value);
        }
        return foundRef;
    }
    finish() {
        // This returns a message we have so far.
        const wm = this._workingMessage;
        wm[0] = [
            utils_1.version,
            this._makeEncodingKeys(),
            this._encodingSpec,
            this._metadata
        ];
        return wm;
    }
}
exports.EncodeCtx = EncodeCtx;
//# sourceMappingURL=encode-context.js.map