"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.typedArrayEncodings = exports.createBinEncoding = exports.sharedArrayBufferEncoding = exports.arrayBufferEncoding = void 0;
const interface_1 = require("../interface");
const utils_1 = require("../utils");
const base64_js_1 = require("base64-js");
const opt_types_1 = require("../opt-types");
const fixed_1 = require("./fixed");
exports.arrayBufferEncoding = {
    name: (0, utils_1.getBuiltInEncodingName)("ArrayBuffer"),
    version: 0,
    [interface_1.fixedIndexProp]: fixed_1.Fixed.ArrayBuffer,
    protos: [ArrayBuffer.prototype].filter(x => !!x),
    encode(input, ctx) {
        const b64 = (0, base64_js_1.fromByteArray)(new Uint8Array(input));
        return b64;
    },
    decoder: {
        create(encodedValue, ctx) {
            const byteArray = (0, base64_js_1.toByteArray)(encodedValue);
            return byteArray.buffer;
        }
    }
};
exports.sharedArrayBufferEncoding = {
    name: (0, utils_1.getBuiltInEncodingName)("SharedArrayBuffer"),
    version: 0,
    [interface_1.fixedIndexProp]: fixed_1.Fixed.SharedArrayBuffer,
    protos: [opt_types_1._SharedArrayBuffer.prototype],
    encode: exports.arrayBufferEncoding.encode.bind(exports.arrayBufferEncoding),
    decoder: {
        create(encodedValue, ctx) {
            // This is not performant, but it's an uncommon use-case.
            const byteArray = (0, base64_js_1.toByteArray)(encodedValue);
            const sharedBuffer = new opt_types_1._SharedArrayBuffer(byteArray.byteLength);
            const sharedByteArray = new Uint8Array(sharedBuffer);
            sharedByteArray.set(byteArray, 0);
            return sharedBuffer;
        }
    }
};
function createBinEncoding(index, ctor) {
    if (!ctor)
        return undefined;
    return {
        name: (0, utils_1.getBuiltInEncodingName)(ctor.name),
        version: 0,
        [interface_1.fixedIndexProp]: index,
        protos: [ctor.prototype],
        encode(input, ctx) {
            return exports.arrayBufferEncoding.encode(input.buffer, ctx);
        },
        decoder: {
            create(encodedValue, ctx) {
                const buffer = exports.arrayBufferEncoding.decoder.create(encodedValue, ctx);
                return new ctor(buffer);
            }
        }
    };
}
exports.createBinEncoding = createBinEncoding;
exports.typedArrayEncodings = [
    createBinEncoding(fixed_1.Fixed.Uint8Array, Uint8Array),
    createBinEncoding(fixed_1.Fixed.Uint8ClampedArray, Uint8ClampedArray),
    createBinEncoding(fixed_1.Fixed.Uint16Array, Uint16Array),
    createBinEncoding(fixed_1.Fixed.Uint32Array, Uint32Array),
    createBinEncoding(fixed_1.Fixed.Int8Array, Int8Array),
    createBinEncoding(fixed_1.Fixed.Int16Array, Int16Array),
    createBinEncoding(fixed_1.Fixed.Int32Array, Int32Array),
    createBinEncoding(fixed_1.Fixed.Float32Array, Float32Array),
    createBinEncoding(fixed_1.Fixed.Float64Array, Float64Array),
    createBinEncoding(fixed_1.Fixed.DataView, DataView),
    createBinEncoding(fixed_1.Fixed.BigInt64Array, opt_types_1._BigInt64Array),
    createBinEncoding(fixed_1.Fixed.BigUint64Array, opt_types_1._BigUint64Array)
]
    .filter(x => !!x)
    .map(x => x);
//# sourceMappingURL=binary.js.map