"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.errorEncodings = exports.createErrorEncoding = void 0;
const interface_1 = require("../interface");
const utils_1 = require("../utils");
const objects_1 = require("./objects");
const fixed_1 = require("./fixed");
const errorProperties = ["stack", "name", "message"];
function createErrorEncoding(index, errorCtor) {
    return {
        protos: [errorCtor.prototype],
        version: 0,
        name: (0, utils_1.getBuiltInEncodingName)(errorCtor.name),
        [interface_1.fixedIndexProp]: index,
        encode(input, ctx) {
            const encodedAsObject = (0, objects_1.encodeObject)(input, ctx, false, errorProperties);
            ctx._isImplicit = false;
            return encodedAsObject;
        },
        decoder: {
            create(encodedValue, ctx) {
                return new errorCtor();
            },
            init(target, encoded, ctx) {
                (0, objects_1.decodeObject)(target, encoded, ctx);
            }
        }
    };
}
exports.createErrorEncoding = createErrorEncoding;
exports.errorEncodings = [
    createErrorEncoding(fixed_1.Fixed.EvalError, EvalError),
    createErrorEncoding(fixed_1.Fixed.RangeError, RangeError),
    createErrorEncoding(fixed_1.Fixed.ReferenceError, ReferenceError),
    createErrorEncoding(fixed_1.Fixed.TypeError, TypeError),
    createErrorEncoding(fixed_1.Fixed.URIError, URIError),
    createErrorEncoding(fixed_1.Fixed.SyntaxError, SyntaxError),
    createErrorEncoding(fixed_1.Fixed.Error, Error)
];
//# sourceMappingURL=errors.js.map