"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.wrapperEncodings = exports.makeWrapperEncoding = exports.dateEncoding = exports.regexpEncoding = void 0;
const interface_1 = require("../interface");
const utils_1 = require("../utils");
const fixed_1 = require("./fixed");
exports.regexpEncoding = {
    protos: [RegExp.prototype],
    version: 0,
    name: (0, utils_1.getBuiltInEncodingName)("RegExp"),
    [interface_1.fixedIndexProp]: fixed_1.Fixed.RegExp,
    encode({ source, flags }, ctx) {
        return flags ? [source, flags] : source;
    },
    decoder: {
        create(input, ctx) {
            if (typeof input === "string") {
                return new RegExp(input);
            }
            else {
                // A bit wasteful, but these shouldn't be common and this avoids having to do parsing.
                return new RegExp(input[0], input[1]);
            }
        }
    }
};
exports.dateEncoding = {
    protos: [Date.prototype],
    version: 0,
    [interface_1.fixedIndexProp]: fixed_1.Fixed.Date,
    name: (0, utils_1.getBuiltInEncodingName)("Date"),
    encode(input, ctx) {
        return input.getTime();
    },
    decoder: {
        create(encodedValue, ctx) {
            return new Date(encodedValue);
        }
    }
};
function makeWrapperEncoding(index, ctor) {
    return {
        name: (0, utils_1.getBuiltInEncodingName)(ctor.name),
        version: 0,
        [interface_1.fixedIndexProp]: index,
        protos: [ctor.prototype],
        encode(input, ctx) {
            return input.valueOf();
        },
        decoder: {
            create(encodedValue, ctx) {
                return new ctor(encodedValue);
            }
        }
    };
}
exports.makeWrapperEncoding = makeWrapperEncoding;
exports.wrapperEncodings = [
    makeWrapperEncoding(fixed_1.Fixed.FundNumber, Number),
    makeWrapperEncoding(fixed_1.Fixed.FundBool, Boolean),
    makeWrapperEncoding(fixed_1.Fixed.FundString, String)
];
//# sourceMappingURL=scalar.js.map