"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.version = exports.isSimpleEncodingSpec = exports.getUnrecognizedSymbol = exports.getUnrecognizedSymbolName = exports.getSymbolName = exports.getClassName = exports.getImplicitClassEncodingName = exports.getBuiltInEncodingName = exports.isNumericString = exports.flatten = exports.defaultsDeep = exports.maxBy = exports.cloneDeep = void 0;
let packageObj;
try {
    packageObj = require("../../package.json");
}
catch (err) {
    packageObj = require("./package.json");
}
function cloneDeep(source) {
    if (typeof source !== "object") {
        return source;
    }
    if (Array.isArray(source)) {
        return source.map(x => cloneDeep(x));
    }
    const newObj = Object.create(Object.getPrototypeOf(source));
    for (const key of Object.keys(source)) {
        newObj[key] = cloneDeep(newObj[key]);
    }
    return newObj;
}
exports.cloneDeep = cloneDeep;
// Based on lodash's implementation: https://github.com/lodash/lodash
function _defaultsDeep(target, source) {
    target = Object(target);
    if (!source)
        return target;
    source = Object(source);
    for (const key of Object.keys(source)) {
        const value = source[key];
        if (typeof target[key] === "object") {
            defaultsDeep(target[key], Object(value));
        }
        else {
            if (target[key] === undefined) {
                target[key] = value;
            }
        }
    }
    return target;
}
function maxBy(target, projection) {
    if (target.length === 0) {
        return undefined;
    }
    let maxProjected = projection(target[0], 0);
    let maxIndex = 0;
    for (let i = 1; i < target.length; i++) {
        const curProjected = projection(target[i], i);
        if (curProjected > maxProjected) {
            maxProjected = curProjected;
            maxIndex = i;
        }
    }
    return target[maxIndex];
}
exports.maxBy = maxBy;
function defaultsDeep(target, ...sources) {
    for (const source of sources) {
        target = _defaultsDeep(target, source);
    }
    return target;
}
exports.defaultsDeep = defaultsDeep;
function flatten(arrs) {
    return [].concat(...arrs);
}
exports.flatten = flatten;
function isNumericString(input) {
    return +input === parseInt(input);
}
exports.isNumericString = isNumericString;
function getBuiltInEncodingName(str) {
    return `/${str}`;
}
exports.getBuiltInEncodingName = getBuiltInEncodingName;
function getImplicitClassEncodingName(str) {
    return str;
}
exports.getImplicitClassEncodingName = getImplicitClassEncodingName;
function getClassName(proto) {
    var _a, _b;
    return (_a = proto[Symbol.toStringTag]) !== null && _a !== void 0 ? _a : (_b = proto.constructor) === null || _b === void 0 ? void 0 : _b.name;
}
exports.getClassName = getClassName;
function getSymbolName(symb) {
    return symb.toString().slice(7, -1);
}
exports.getSymbolName = getSymbolName;
function getUnrecognizedSymbolName(name) {
    return `preszr unknown: ${name}`;
}
exports.getUnrecognizedSymbolName = getUnrecognizedSymbolName;
function getUnrecognizedSymbol(name) {
    return Symbol(getUnrecognizedSymbolName(name));
}
exports.getUnrecognizedSymbol = getUnrecognizedSymbol;
function isSimpleEncodingSpec(candidate) {
    return typeof candidate === "symbol" || typeof candidate === "function";
}
exports.isSimpleEncodingSpec = isSimpleEncodingSpec;
exports.version = packageObj.version.split(".")[0];
//# sourceMappingURL=utils.js.map