"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const ava_1 = __importDefault(require("ava"));
const utils_1 = require("@lib/utils");
const utils_2 = require("../utils");
const utils_3 = require("@lib/encodings/utils");
const testSymbol = Symbol("test");
class TestClass {
    constructor() {
        this.field = 5;
    }
}
class TestClass2 {
    constructor() {
        this.field = 10;
    }
}
// This is so deepEqual won't just treat the classes as {}
TestClass.prototype.field = 5;
TestClass2.prototype.field = 10;
(0, ava_1.default)("implicit class encoding name", t => {
    t.is((0, utils_1.getBuiltInEncodingName)("test"), "/test");
});
(0, ava_1.default)("from symbol with name", t => {
    const encoding = (0, utils_3.makeFullEncoding)(testSymbol);
    t.deepEqual(encoding, {
        name: "test",
        symbol: testSymbol
    });
});
(0, ava_1.default)("error when trying with symbol without name", t => {
    // eslint-disable-next-line symbol-description
    const err = t.throws(() => (0, utils_3.makeFullEncoding)(Symbol()));
    t.true(err.message.includes(`no name`));
});
(0, ava_1.default)("symbol encoding with explicit name unchanged", t => {
    const encoding = {
        name: "a",
        symbol: testSymbol
    };
    t.deepEqual(encoding, (0, utils_3.makeFullEncoding)(encoding));
});
(0, ava_1.default)("encoding from class", t => {
    const encoding = (0, utils_3.makeFullEncoding)(TestClass);
    t.is(encoding.name, "TestClass");
    t.is(encoding.protos.length, 1);
    t.is(encoding.protos[0], TestClass.prototype);
    const dummyCtx = (0, utils_2.getDummyCtx)();
    const result = encoding.encode(new TestClass(), dummyCtx);
    t.deepEqual(result, { field: 5 });
});
(0, ava_1.default)("encoding from prototype", t => {
    const encoding = (0, utils_3.makeFullEncoding)({
        proto: TestClass.prototype
    });
    t.is(encoding.name, "TestClass");
    t.is(encoding.protos.length, 1);
    const [p1] = encoding.protos;
    t.is(p1, TestClass.prototype);
    const dummyCtx = (0, utils_2.getDummyCtx)();
    const result = encoding.encode(new TestClass(), dummyCtx);
    t.deepEqual(result, { field: 5 });
});
(0, ava_1.default)("encoding with multiple prototypes", t => {
    const f = () => 1;
    const encoding = (0, utils_3.makeFullEncoding)({
        protos: [class {
            }, TestClass.prototype],
        version: 5,
        name: "blah",
        encode: f,
        decoder: {
            create: f
        }
    });
    t.is(encoding.name, "blah");
    t.is(encoding.version, 5);
    t.is(encoding.decoder.create, f);
    t.is(encoding.encode, f);
});
(0, ava_1.default)("encoding prototype field", t => {
    const encoding = (0, utils_3.makeFullEncoding)({
        proto: TestClass.prototype
    });
    t.is(encoding.name, (0, utils_1.getImplicitClassEncodingName)("TestClass"));
    t.deepEqual(encoding.decoder.create({}, {}), Object.create(TestClass.prototype));
});
(0, ava_1.default)("error - nameless ctor without key", t => {
    const err = t.throws(() => (0, utils_3.makeFullEncoding)(class {
    }));
    t.true(err.message.includes("no name"));
});
(0, ava_1.default)("error - cannot get prototype from ctor", t => {
    const brokenCtor = function () { };
    brokenCtor.prototype = null;
    const err = t.throws(() => (0, utils_3.makeFullEncoding)(brokenCtor));
    t.regex(err.message, /prototype from constructor/);
});
(0, ava_1.default)("error - multiple prototypes provide key", t => {
    const err = t.throws(() => (0, utils_3.makeFullEncoding)({
        prototypes: [{}, {}],
        encode: (() => { }),
        decoder: {}
    }));
    t.regex(err.message, /provide a key/);
});
(0, ava_1.default)("error - multiple prototypes, provide encoder/decoder", t => {
    const err = t.throws(() => (0, utils_3.makeFullEncoding)({
        prototypes: [{}, {}],
        key: "blah"
    }));
    t.regex(err.message, /decoder.*encode/);
});
(0, ava_1.default)("error - no prototype(s)", t => {
    const err = t.throws(() => (0, utils_3.makeFullEncoding)({}));
    t.regex(err.message, /specify prototypes/);
});
//# sourceMappingURL=encoding-makers.js.map