"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const ava_1 = __importDefault(require("ava"));
const _lib_1 = require("@lib");
const utils_1 = require("../../lib/utils");
(0, ava_1.default)("encoding - entity is array", t => {
    t.true(Array.isArray((0, _lib_1.encode)({})));
});
(0, ava_1.default)("decoding - header structure", t => {
    const [header] = (0, _lib_1.encode)({});
    const [version, encodingKeys, encodingSpec, metadata] = header;
    t.is(version, utils_1.version);
    t.deepEqual(encodingKeys, []);
    t.deepEqual(encodingSpec, {});
    t.deepEqual(metadata, {});
});
function isBadPayloadError(err) {
    return err.message.includes("not preszr-encoded");
}
function isBadVersionError(err) {
    return err.message.includes("version");
}
(0, ava_1.default)("decoding - error when trying to decode anomalous object", t => {
    const badPayloads = [
        {},
        [],
        [1],
        [[]],
        [[utils_1.version, [], {}, {}]],
        [["g", {}, {}], 1],
        [[1, {}, {}], 1],
        [[utils_1.version], 1],
        [[utils_1.version, {}], 1] // no metadata,
    ];
    for (const payload of badPayloads) {
        const err = t.throws(() => (0, _lib_1.decode)(payload));
        t.true(isBadPayloadError(err));
    }
});
(0, ava_1.default)("decoding - error when trying to decode wrong version", t => {
    const encoded = [[utils_1.version + 1, [], {}, {}], {}];
    const err = t.throws(() => (0, _lib_1.decode)(encoded));
    t.true(isBadVersionError(err));
});
(0, ava_1.default)("decoding error - unknown encoding", t => {
    const encoded = [[utils_1.version, ["test.v0"], { 1: 0 }, {}], 0];
    const err = t.throws(() => (0, _lib_1.decode)(encoded));
    t.regex(err.message, /no prototype encoding/);
});
//# sourceMappingURL=entity-basics.js.map