"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.fullErrorEquality = void 0;
const ava_1 = __importDefault(require("ava"));
const tools_1 = require("../tools");
const fixed_1 = require("@lib/encodings/fixed");
const default_1 = require("@lib/default");
exports.fullErrorEquality = (0, tools_1.testBuilder)()
    .title(({ original }) => `Error Equality: ${original.constructor.name}`)
    .eqAssertion((t, decoded, original) => {
    t.deepEqual(decoded, original, "DECODED != ORIGINAL");
    t.like(decoded, {
        message: original.message,
        stack: original.stack,
        name: original.name
    }, "DECODED != ORIGINAL");
});
const checkErrorMacro = exports.fullErrorEquality.instance(default_1.defaultPreszr);
{
    const regularError = new Error("hi");
    (0, ava_1.default)(checkErrorMacro.getSimple(), regularError, (0, tools_1.preszr)((0, tools_1.encoded)({
        stack: "2",
        name: "3",
        message: "4"
    }, fixed_1.Fixed.Error), (0, tools_1.items)(regularError.stack, regularError.name, regularError.message)));
}
const encodedError = (err, type) => {
    return (0, tools_1.preszr)((0, tools_1.encoded)({
        stack: "2",
        name: "3",
        message: "4"
    }, type), (0, tools_1.items)(err.stack, err.name, err.message));
};
{
    const syntaxError = new SyntaxError("A SyntaxError");
    const typeError = new TypeError("A TypeError");
    const uriError = new URIError("A URIError");
    const evalError = new EvalError("A EvalError");
    const refError = new ReferenceError("A ReferenceError");
    const rangeError = new RangeError("A RangeError");
    (0, ava_1.default)(checkErrorMacro.getSimple(), syntaxError, encodedError(syntaxError, fixed_1.Fixed.SyntaxError));
    (0, ava_1.default)(checkErrorMacro.getSimple(), typeError, encodedError(typeError, fixed_1.Fixed.TypeError));
    (0, ava_1.default)(checkErrorMacro.getSimple(), uriError, encodedError(uriError, fixed_1.Fixed.URIError));
    (0, ava_1.default)(checkErrorMacro.getSimple(), evalError, encodedError(evalError, fixed_1.Fixed.EvalError));
    (0, ava_1.default)(checkErrorMacro.getSimple(), refError, encodedError(refError, fixed_1.Fixed.ReferenceError));
    (0, ava_1.default)(checkErrorMacro.getSimple(), rangeError, encodedError(rangeError, fixed_1.Fixed.RangeError));
}
function assignError(x) {
    return Object.assign(new Error(), {
        ...x,
        message: x.message,
        stack: x.stack,
        name: x.name
    });
}
{
    class SubError extends Error {
        constructor() {
            super(...arguments);
            this.name = "SubError";
            this.newKey = "x";
        }
    }
    SubError.prototype.protoKey = "protoKey";
    SubError.prototype.name = "SubError";
    const err3 = new SubError("test");
    (0, ava_1.default)(checkErrorMacro.get(), {
        original: err3,
        encoded: (0, tools_1.preszr)((0, tools_1.encoded)({
            stack: "4",
            name: "2",
            message: "5",
            newKey: "3"
        }, fixed_1.Fixed.Error), (0, tools_1.items)("SubError", "x", err3.stack, "test")),
        decoded: assignError(err3)
    });
}
//# sourceMappingURL=error.js.map