"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const ava_1 = __importDefault(require("ava"));
const _lib_1 = require("@lib");
const tools_1 = require("../tools");
const fixed_1 = require("@lib/encodings/fixed");
(0, ava_1.default)("invalid definition - has name", t => {
    t.throws(() => (0, _lib_1.Preszr)([
        { proto: Date.prototype, name: "should_be_empty", version: 100 }
    ]));
});
(0, ava_1.default)("Invalid definition version is 0", t => {
    t.throws(() => {
        (0, _lib_1.Preszr)([{ proto: Date.prototype, version: 0 }]);
    });
});
(0, ava_1.default)("Passes when valid", t => {
    t.notThrows(() => (0, _lib_1.Preszr)([{ proto: Date.prototype }]));
});
(0, ava_1.default)("modified Date is detected", t => {
    const time = Date.now();
    const a = new Date(time);
    const b = new Date(time);
    Object.assign(a, { a: 1 });
    t.notDeepEqual(a, b);
    Object.assign(b, { a: 1 });
    t.deepEqual(a, b);
});
function getNewDateEncoding(version, magicKey) {
    return {
        proto: Date.prototype,
        version,
        encode(input, ctx) {
            const arr = [input.getTime()];
            arr.push(input[magicKey]);
            return arr;
        },
        decoder: {
            create(encoded, ctx) {
                return new Date(encoded[0]);
            },
            init(target, encoded, ctx) {
                target[magicKey] = ctx.decode(encoded[1]);
            }
        }
    };
}
const modifiedDateEncoding = getNewDateEncoding(1, "a");
const refDate = new Date();
{
    const testWithModifiedDates = (0, tools_1.testBuilder)((0, _lib_1.Preszr)([modifiedDateEncoding])).get();
    const testObject = Object.assign(new Date(refDate), {
        a: 100
    });
    (0, ava_1.default)("Override is used instead of default", testWithModifiedDates, {
        original: testObject,
        encoded: (0, tools_1.preszr)((0, tools_1.encoded)([testObject.getTime(), 100], fixed_1.Fixed.Date))
    });
}
const modifiedDateEnc2 = getNewDateEncoding(2, "b");
{
    const testWith2Overrides = (0, tools_1.testBuilder)((0, _lib_1.Preszr)([modifiedDateEnc2, modifiedDateEncoding])).get();
    (0, ava_1.default)("2nd override used", testWith2Overrides, {
        original: Object.assign(new Date(refDate), { b: 100 }),
        encoded: (0, tools_1.preszr)((0, tools_1.encoded)([refDate.getTime(), 100], fixed_1.Fixed.Date))
    });
}
//# sourceMappingURL=override-default.js.map