"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
/* tslint:disable:no-construct */
const ava_1 = __importDefault(require("ava"));
const utils_1 = require("../utils");
const _lib_1 = require("@lib");
const tools_1 = require("../tools");
const fixed_1 = require("@lib/encodings/fixed");
class ParentClass {
    constructor(obj = {}) {
        Object.assign(this, obj);
    }
}
class ChildClass extends ParentClass {
}
const parentClassEncodingName = (0, utils_1.getImplicitClassEncodingKey)("ParentClass");
const childClassEncodingName = (0, utils_1.getImplicitClassEncodingKey)("ChildClass");
(0, ava_1.default)("deepEqual distinguishes prototypes", t => {
    t.notDeepEqual(new ParentClass(), {});
    t.notDeepEqual(new ChildClass(), {});
    t.notDeepEqual(new ParentClass(), new ChildClass());
});
const builder = (0, tools_1.testBuilder)();
// Unknown TestClass encoded as regular object
(0, ava_1.default)("Unknown prototype", builder.get(), {
    original: new ParentClass(),
    encoded: (0, tools_1.preszr)((0, tools_1.items)({})),
    // Decoded output expected to be different from original:
    decoded: {}
});
{
    const testWithParent = builder.instance((0, _lib_1.Preszr)([ParentClass])).get();
    (0, ava_1.default)("known prototype", testWithParent, {
        original: new ParentClass({ a: 1 }),
        encoded: (0, tools_1.preszr)((0, tools_1.encoded)({ a: 1 }, parentClassEncodingName))
        // Here decoded output is supposed to be the same
    });
    (0, ava_1.default)("subclass of known prototype", testWithParent, {
        original: new ChildClass({ a: 2 }),
        encoded: (0, tools_1.preszr)((0, tools_1.encoded)({ a: 2 }, parentClassEncodingName)),
        decoded: new ParentClass({ a: 2 })
    });
}
{
    const testWithBoth = builder
        .instance((0, _lib_1.Preszr)([ParentClass, ChildClass]))
        .get();
    (0, ava_1.default)("both classes", testWithBoth, {
        original: { a: new ParentClass({ a: 1 }), b: new ChildClass({ a: 2 }) },
        encoded: (0, tools_1.preszr)((0, tools_1.items)({ a: "2", b: "3" }), (0, tools_1.encoded)({ a: 1 }, parentClassEncodingName), (0, tools_1.encoded)({ a: 2 }, childClassEncodingName))
    });
    class ChildChildClass extends ChildClass {
    }
    (0, ava_1.default)("unknown child child class matches nearest", testWithBoth, {
        original: new ChildChildClass({ a: 3 }),
        encoded: (0, tools_1.preszr)((0, tools_1.encoded)({ a: 3 }, childClassEncodingName)),
        decoded: new ChildClass({ a: 3 })
    });
}
{
    const testForNullProtoObject = builder.eqAssertion((t, decoded, original) => {
        t.deepEqual(decoded, original);
        t.is(Object.getPrototypeOf(decoded), null);
    });
    (0, ava_1.default)("null prototype object", testForNullProtoObject.get(), {
        original: Object.create(null, {
            a: {
                value: 5,
                enumerable: true
            }
        }),
        encoded: (0, tools_1.preszr)((0, tools_1.encoded)({ a: 5 }, fixed_1.Fixed.NullProto))
    });
}
//# sourceMappingURL=prototype-matching.js.map