"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.testBuilder = exports.GenericTestBuilder = void 0;
const ava_1 = __importDefault(require("ava"));
const base_test_builder_1 = require("./base-test-builder");
class GenericTestBuilder extends base_test_builder_1.TestBuilder {
    eqAssertion(eq) {
        return this.with({
            _equalityAssertion: eq
        });
    }
    _test(t, args) {
        const { instance } = args;
        const toEncoded = instance.encode(args.original);
        t.deepEqual(toEncoded, args.encoded, "REAL_ENCODED != EXPECTED_ENCODED");
        const decoded = "decoded" in args ? args.decoded : args.original;
        const realDecoded = instance.decode(toEncoded);
        if (this._equalityAssertion) {
            this._equalityAssertion(t, realDecoded, decoded);
        }
        else {
            t.deepEqual(realDecoded, decoded, "REAL_DECODED != EXPECTED_DECODED");
        }
    }
    getSimple() {
        const self = this;
        const originalMacro = self.get();
        return ava_1.default.macro({
            title(title, original, encoded) {
                return originalMacro.title(title, {
                    original,
                    encoded
                });
            },
            exec(t, original, encoded) {
                return originalMacro.exec(t, {
                    encoded,
                    original
                });
            }
        });
    }
}
exports.GenericTestBuilder = GenericTestBuilder;
function testBuilder(instance) {
    return new GenericTestBuilder().instance(instance);
}
exports.testBuilder = testBuilder;
//# sourceMappingURL=test-builder.js.map