"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.toBase64 = exports.encodeDecodeMacro = exports.getDummyCtx = exports.combAttachHeader = exports.testDecodeMacroBindPreszr = exports.testEncodeMacroBindPreszr = exports.testDecodeMacro = exports.testEncodeMacro = exports.createWithTitle = exports.preszrDefaultHeader = exports.createPreszrRep = exports.simplifyEncoding = exports.getEncodingComponent = exports.embedPreszrVersion = exports.createSharedArrayBuffer = exports.createArrayBuffer = exports.createSparseArray = exports.getImplicitClassEncodingKey = exports.stringify = void 0;
const utils_1 = require("@lib/utils");
const lodash_1 = require("lodash");
const core_1 = require("@lib/core");
const stringify = (value) => {
    if (typeof value === "object") {
        return JSON.stringify(value);
    }
    if (typeof value === "bigint") {
        return `${value}n`;
    }
    if (Object.is(value, -0)) {
        return "-0";
    }
    return `${value}`;
};
exports.stringify = stringify;
function getImplicitClassEncodingKey(cls) {
    return `${cls}.v0`;
}
exports.getImplicitClassEncodingKey = getImplicitClassEncodingKey;
function createSparseArray(arrayLikeObj) {
    const arr = [];
    for (const [key, value] of Object.entries(arrayLikeObj)) {
        arr[key] = value;
    }
    return arr;
}
exports.createSparseArray = createSparseArray;
function createArrayBuffer(...bytes) {
    const arr = new Uint8Array(bytes);
    return arr.buffer;
}
exports.createArrayBuffer = createArrayBuffer;
function createSharedArrayBuffer(...bytes) {
    const sharedArrayBuffer = new SharedArrayBuffer(bytes.length);
    const asBytes = new Uint8Array(sharedArrayBuffer);
    for (let i = 0; i < asBytes.length; i++) {
        asBytes[i] = bytes[i];
    }
    return asBytes.buffer;
}
exports.createSharedArrayBuffer = createSharedArrayBuffer;
function embedPreszrVersion(encoded) {
    encoded = (0, lodash_1.cloneDeep)(encoded);
    const encodingSpec = encoded[0].shift();
    encoded[0].unshift(utils_1.version, ...getEncodingComponent(encodingSpec));
    return encoded;
}
exports.embedPreszrVersion = embedPreszrVersion;
function getEncodingComponent(encodingSpec) {
    const encodingKeys = [];
    for (const [, value] of Object.entries(encodingSpec)) {
        if (!encodingKeys.includes(value)) {
            encodingKeys.push(value);
        }
    }
    for (const [key, value] of Object.entries(encodingSpec)) {
        encodingSpec[key] = encodingKeys.indexOf(value);
    }
    return [encodingKeys, encodingSpec];
}
exports.getEncodingComponent = getEncodingComponent;
function simplifyEncoding(encoding) {
    const clone = (0, lodash_1.cloneDeep)(encoding);
    const [[, keys, info]] = clone;
    for (const [k, v] of Object.entries(info)) {
        info[k] = keys[v];
    }
    clone[0].splice(1, 1);
    return clone;
}
exports.simplifyEncoding = simplifyEncoding;
function createPreszrRep([encodingSpec, meta], ...arr) {
    const header = [
        utils_1.version,
        ...getEncodingComponent(encodingSpec),
        meta
    ];
    return [header, ...arr];
}
exports.createPreszrRep = createPreszrRep;
function preszrDefaultHeader(...arr) {
    return createPreszrRep([{}, {}], ...arr);
}
exports.preszrDefaultHeader = preszrDefaultHeader;
function createWithTitle(macro, argsFunc, titleFunc) {
    const newMacro = (t, ...args) => macro(t, ...argsFunc(...args));
    newMacro.title = titleFunc;
    return newMacro;
}
exports.createWithTitle = createWithTitle;
const defaultPreszr = new core_1.Preszr();
const testEncodeMacro = (t, decoded, encoded, preszr = defaultPreszr) => {
    const rEncoded = preszr.encode(decoded);
    t.deepEqual(simplifyEncoding(rEncoded), simplifyEncoding(encoded));
};
exports.testEncodeMacro = testEncodeMacro;
const testDecodeMacro = (t, decoded, encoded, preszr = defaultPreszr) => {
    const rDecoded = preszr.decode(encoded);
    t.deepEqual(rDecoded, decoded);
};
exports.testDecodeMacro = testDecodeMacro;
const testEncodeMacroBindPreszr = preszr => (a, b, c) => (0, exports.testEncodeMacro)(a, b, c, preszr);
exports.testEncodeMacroBindPreszr = testEncodeMacroBindPreszr;
const testDecodeMacroBindPreszr = preszr => (a, b, c) => (0, exports.testDecodeMacro)(a, b, c, preszr);
exports.testDecodeMacroBindPreszr = testDecodeMacroBindPreszr;
const combAttachHeader = titleFunc => {
    const attachHeader = (decoded, encoded) => [
        decoded,
        preszrDefaultHeader(...encoded)
    ];
    return [
        createWithTitle(exports.testEncodeMacro, attachHeader, (title, ...args) => `encode:: ${title ?? titleFunc(...args)}`),
        createWithTitle(exports.testDecodeMacro, attachHeader, (title, ...args) => `decode:: ${title ?? titleFunc(...args)}`)
    ];
};
exports.combAttachHeader = combAttachHeader;
function getDummyCtx() {
    return {
        encode: x => x,
        decode: x => x
    };
}
exports.getDummyCtx = getDummyCtx;
const encodeDecodeMacro = (args) => {
    return [
        createWithTitle(args.encode, (decoded, encoded, preszr) => [
            decoded,
            embedPreszrVersion(encoded),
            preszr
        ], title => `encode :: ${title}`),
        createWithTitle(args.decode, (decoded, encoded, preszr) => [
            decoded,
            embedPreszrVersion(encoded),
            preszr
        ], title => `decode :: ${title}`)
    ];
};
exports.encodeDecodeMacro = encodeDecodeMacro;
function toBase64(buf) {
    return Buffer.from(buf).toString("base64");
}
exports.toBase64 = toBase64;
//# sourceMappingURL=utils.js.map